/**
 * @file HL7800_???.ino
 * @brief HL7800 Library test-app <br>
 * 
 * @date  2019/9/5 Preview-Alpha
 * @author kt-nakamura@kddi-tech.com
 */
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
#include "Arduino.h"
#include "common.h"
#include "hl7800_client.h"
#include "hl7800_udp.h"
#include "hl7800_http.h"
#include "hl7800_modem.h"
#include "hl7800_serial.h"
#include "hl7800_leaf.h"
#include "app_config.h"

// #include <Adafruit_SleepyDog.h> // DeepSleep https://github.com/adafruit/Adafruit_SleepyDog

#include <Wire.h>


HL7800Client client_hl78;
HL7800Udp udp_hl78;
HL7800Http http_hl78;
HL7800Modem modem_hl78;
HL7800Leaf leaf_hl78;
extern HL7800Serial serial_hl78;

enum PdpConnectState { 
  RESET,
  INIT,
  CONNECTED,
  FAIL
}; 
PdpConnectState ConnectState = RESET; 

/**
 * LTE 自動接続タスク<br>
 * モデムの初期化からPDP接続までを自動的に行う
 * @param なし
 * @return  true:接続中, false:失敗
 */
bool connect_run(void) {
  if (ConnectState == RESET) {
    if (!modem_hl78.Init()) {
      // モデムからの応答がない場合はエラー終了
      DBGSERIAL.println("@@@@@@ Modem not resoponed!");
      return false;
    }

    // SIM PINチェック
    if (!modem_hl78.chkPin()) {
      // 利用可能なSIMが見つからない場合はエラー終了
      DBGSERIAL.print("PIN locked!");
      ConnectState = RESET;  // 初期化からやり直す
      return false;
    }
    // モデム初期化OK
    ConnectState = INIT;
  }

  // RSSIレベルをチェックし在圏確認
  int rssi = modem_hl78.getRssi();
  if (rssi == 99) {
    // 圏外の場合はエラー終了としし再試行させる
    DBGSERIAL.print("invalid RSSI = ");
    DBGSERIAL.println(rssi);
    return false;
  }
  DBGSERIAL.print("RSSI = ");
  DBGSERIAL.println(rssi);

  // ネットワーク未接続の場合はPDPアクティベートする
  if (!modem_hl78.isActive()) {
    DBGSERIAL.println("PDP connectiong ..");
    if (modem_hl78.activatePdp(PDP_APN, PDP_USER, PDP_PASSWORD, MODEM_CONNECT_TIMEOUT)) {
      DBGSERIAL.println("PDP Connected");
      ConnectState = CONNECTED;
    } else {
      // ネットワーク接続失敗の場合はエラー終了し再試行させる     
      DBGSERIAL.println("PDP Connection Failed!");
      ConnectState = RESET;  // 初期化からやり直す
      return false;
    }
  }

  // 自IPアドレスをチェックしてネットワーク死活チェック
  if (!modem_hl78.chkIpAddress()) {
   // 自IPが不正の場合はPDP切断したうえでエラー終了し、モデムリセットから再試行      
    DBGSERIAL.println("Invalid IP -- PDP disconnect!");
    modem_hl78.deactivate();
    ConnectState = RESET;  // 初期化からやり直す
    return false;
  }

  return true;
}

/**
 * Arduinoアプリ初期化<br>
 * @param buffer 受け取りデータバッファ
 * @param len 受信したいデータ数
 * @param なし
 * @return  true:成功, false:失敗
 * @return  1:成功, 0:失敗
 * @return  受信済みデータバイト数
 * @return  受信データ(NULL終端)
 * @return なし
 */
void setup() {
  // put your setup code here, to run once:
  ConnectState = RESET; 

  DBGSERIAL.begin(DBGBPS);
#ifdef USE_USBSERIAL
  while (!DBGSERIAL) {
    ; // wait for USB port to connect. Needed for Arduino M0 only
  }
#endif
  delay(1000);
  DBGSERIAL.println("startup...");

  Wire.begin();
  leaf_hl78.setLeafPower(true);
  delay(1000);

  pinMode(LED_BUILTIN, OUTPUT);
}


// socket test (HTTP GET)
void socket_http_test() {
  DBGSERIAL.println("test: TCP HTTP-GET");
  // TCPソケットをオープン
  byte server[] = { TEST_SERVER}; 
  if (!client_hl78.connect(server,80)) {
//  if (!client_hl78.connect(TEST_URL,80)) {
    DBGSERIAL.println("TCP connection failed!");
  } else {
    DBGSERIAL.println("TCP connected");

    // TCP 送信
    if (client_hl78.print("GET /get_29.txt HTTP/1.0\r\nConnection: Keep-Alive\r\n\r\n")) { // Keep-Aliveをしてしないと最終パケットで切断されデータが受け取れない
//    if (client_hl78.print("GET /get37.txt HTTP/1.0\r\n\r\n")) {
      DBGSERIAL.println("TCP write failed");
    } else {
      // TCP 受信
      while (client_hl78.available() > 0) { // HTTPレスポンスヘッダを意識していないためモデムのTCP切断で終了
        char c = client_hl78.read();
        DBGSERIAL.write(c);
      }
    }
    // TCPソケットをクローズ
    client_hl78.stop();
  }
}

// UDPテスト
bool udp_init = false;
void udp_test() {
  DBGSERIAL.println("@@@@@ test: UDP");
  uint16_t localPort = 9999;      // local port to listen on
  unsigned int remotePort = 8888;      // xmit remote port
  byte remoteIp[] = { TEST_SERVER }; 

  // UDPライブラリを初期化
  if (udp_hl78.begin(localPort)) {
    DBGSERIAL.println("@@@@@@ UDP init sucess");
    udp_init = true;
  } else {
    // 失敗した場合は関数から抜ける
    DBGSERIAL.println("@@@@@@ UDP init fail");
    return;
  }
  // テストサーバに文字列をUDP送信
  udp_hl78.beginPacket(remoteIp, remotePort);
  udp_hl78.print("hello1234");
  udp_hl78.endPacket();

  // テストサーバからの応答待ち
  // ※KDDIのネットワークでは非セッションのパケットは到達しないため動作しない
  // $ nc -u 10.224.112.79 9999
  // test <CRLF>
//  DBGSERIAL.println("wait UDP packet");
//  while (1) {
//    int packetSize = udp_hl78.parsePacket();
//  }
  
//  udp_hl78.stop();
  
}

char HttpBuff[RCVBUFF_SIZE]; // HTTP テストで使う受信メッセージバッファ

/**
 * Arduinoアプリ初期化<br>
 * @param buffer 受け取りデータバッファ
 * @param len 受信したいデータ数
 * @param なし
 * @return  true:成功, false:失敗
 * @return  1:成功, 0:失敗
 * @return  受信済みデータバイト数
 * @return  受信データ(NULL終端)
 * @return なし
 */
// HTTPでデータを取得(GET)
void http_get_test() {
  DBGSERIAL.println("@@@@@ test: HTTP GET");
  byte server[] = { TEST_SERVER };
  memset(HttpBuff,0x0,RCVBUFF_SIZE); 
  if (!http_hl78.connect(server,HTTP_PORT, USE_HTTP, NULL, NULL)) {
//  if (!http_hl78.connect(TEST_URL, HTTP_PORT, USE_HTTP, NULL, NULL)) {
    DBGSERIAL.println("@@@@@@ HTTP connection failed!");
  } else {
    DBGSERIAL.println("@@@@@@ HTTP connected");
    int resp = http_hl78.get(TEST_URI_GET, HttpBuff, false);
    DBGSERIAL.print("@@@@@@ HTTP Resp = ");
    DBGSERIAL.println(resp);
    if (resp > 0) {
      DBGSERIAL.println("@@@@@@ HTTP data");
      DBGSERIAL.println("======================================");
      DBGSERIAL.println(HttpBuff);
      DBGSERIAL.println("======================================");
    }
    http_hl78.close();
  }
}

/**
 * Arduinoアプリ初期化<br>
 * @param buffer 受け取りデータバッファ
 * @param len 受信したいデータ数
 * @param なし
 * @return  true:成功, false:失敗
 * @return  1:成功, 0:失敗
 * @return  受信済みデータバイト数
 * @return  受信データ(NULL終端)
 * @return なし
 */
// HTTPでデータを取得(HEAD)
void http_head_test() {
  DBGSERIAL.println("@@@@@ test: HTTP HEAD");
  byte server[] = { TEST_SERVER }; 
  memset(HttpBuff,0x0,RCVBUFF_SIZE); 
  if (!http_hl78.connect(server,HTTP_PORT, USE_HTTP, NULL, NULL)) {
//  if (!http_hl78.connect(TEST_URL, HTTP_PORT, USE_HTTP, NULL, NULL)) {
    DBGSERIAL.println("@@@@@@ HTTP connection failed!");
  } else {
    DBGSERIAL.println("@@@@@@ HTTP connected");
    int resp = http_hl78.head("/get37.txt", HttpBuff);
    DBGSERIAL.print("@@@@@@ HTTP Resp = ");
    DBGSERIAL.println(resp);
    if (resp > 0) {
      DBGSERIAL.println("@@@@@@ HTTP data");
      DBGSERIAL.println("======================================");
      DBGSERIAL.println(HttpBuff);
      DBGSERIAL.println("======================================");
    }
    http_hl78.close();
  }
}

/**
 * HTTPでデータ送信/取得(POST)<br>
 * @param なし
 * @return なし
 */
void http_post_test() {
  DBGSERIAL.println("@@@@@ test: HTTP POST");
  byte server[] = { TEST_SERVER }; 
  memset(HttpBuff,0x0,RCVBUFF_SIZE); 
//  if (!http_hl78.connect(server,HTTP_PORT, USE_HTTP, NULL, NULL)) {
  if (!http_hl78.connect(TEST_URL, HTTP_PORT, USE_HTTP, NULL, NULL)) {
    DBGSERIAL.println("@@@@@@ HTTP connection failed!");
  } else {
    DBGSERIAL.println("@@@@@@ HTTP connected");
    int resp = http_hl78.post("/api.php?num=10", HttpBuff, false);
    DBGSERIAL.print("@@@@@@ HTTP Resp = ");
    DBGSERIAL.println(resp);
    if (resp > 0) {
      DBGSERIAL.println("@@@@@@ HTTP data");
      DBGSERIAL.println("======================================");
      DBGSERIAL.println(HttpBuff);
      DBGSERIAL.println("======================================");
    }
    http_hl78.close();
  }
}

/**
 * HTTPでデータ送信/取得(PUT)<br>
 * @param なし
 * @return なし
 */
void http_put_test() {
  DBGSERIAL.println("@@@@@ test: HTTP PUT");
  char sendbuff[60];
  byte server[] = { TEST_SERVER }; 
  memset(HttpBuff,0x0,RCVBUFF_SIZE); 

  if (!http_hl78.connect(server,HTTP_PORT, USE_HTTP, NULL, NULL)) {
//  if (!http_hl78.connect(TEST_URL, HTTP_PORT, USE_HTTP, NULL, NULL)) {
    DBGSERIAL.println("@@@@@@ HTTP connection failed!");
  } else {
    DBGSERIAL.println("@@@@@@ HTTP connected");

    sprintf(sendbuff,"Except: 100-continue\r\n");
//    sprintf(sendbuff,"Content-Length: 16\r\nExcept: 100-continue\r\n");
    http_hl78.config(sendbuff);
    
    sprintf(sendbuff,"0123456789ABCDEF\r\n");
    int resp = http_hl78.put("/puttest/put.php", sendbuff, HttpBuff, false);
    DBGSERIAL.print("@@@@@@ HTTP Resp = ");
    DBGSERIAL.println(resp);
    if (resp > 0) {
      DBGSERIAL.println("@@@@@@ HTTP data");
      DBGSERIAL.println("======================================");
      DBGSERIAL.println(HttpBuff);
      DBGSERIAL.println("======================================");
    }
    http_hl78.close();
  }
}

void api_test(void) {
#if 0
  int mvolt =  leaf_hl78.getLeafADC();
  DBGSERIAL.print("@@@@@@ batt=");
  DBGSERIAL.print(mvolt);
  DBGSERIAL.println("mV");
  delay(1000);
#endif
  // IMEI取得チェック
  char imei[20];
  if (modem_hl78.getImei(imei) > 0) {
    DBGSERIAL.print("@@@@@@ IMEI = ");
    DBGSERIAL.println(imei);
  }
  // CCID取得チェック
  char ccid[20];
  if (modem_hl78.getCcid(ccid) > 0) {
    DBGSERIAL.print("@@@@@@ CCID = ");
    DBGSERIAL.println(ccid);
  }

  // eDRX
  modem_hl78.setEDRX(-1);
}

/**
 * Arduinoアプリタスク<br>
 * @param なし
 * @return なし
 */
void loop() {
  // put your main code here, to run repeatedly:
  DBGSERIAL.println("@@@@ loop() start");
  digitalWrite(LED_BUILTIN, HIGH);

#ifdef USE_SLEEP
  // モデムの省電力状態を解除
  modem_hl78.leaveSleep();
#endif

#ifdef USE_POWEROFF
  // モデム電源オン
  leaf_hl78.setLeafPower(true);
#endif

#ifdef USE_POWERDOWN
  // モデム電源復帰
  modem_hl78.powerUp();
#endif

  // LTE 自動接続タスク
  if (!connect_run()) {
    // 失敗時はloop()から抜けて再試行
    delay(1000);
    return;
  }

  // APIテスト
//  socket_http_test();
//  udp_test();
//  http_head_test();
  http_get_test();
//  http_post_test();
//  http_put_test();

  // テスト成功後はPDP接続を切断
  ConnectState = INIT;
  modem_hl78.deactivate();
  
#ifdef USE_SLEEP
  // モデムを省電力状態に移行
  modem_hl78.enterSleep(SLEEP_MODE);
#endif

#ifdef USE_POWERDOWN
  // モデムパワーダウン
  modem_hl78.powerDown();
  ConnectState = RESET;
#endif

#ifdef USE_POWEROFF
  // モデム電源断
  leaf_hl78.setLeafPower(false);
  ConnectState = RESET;
#endif
 
  digitalWrite(LED_BUILTIN, LOW);
  DBGSERIAL.println("@@@@ loop()end");
  delay(30000); // データを流しすぎないよう待機

//  int sleepMS = Watchdog.sleep(30000);
//  USBDevice.attach();

}
