/**
 * @file HttpSample.ino
 * @brief HL7800 Library test-app <br>
 * 
 * @date  2019/9/17 Preview-Beta
 * @author kt-nakamura@kddi-tech.com
 */
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
#include "Arduino.h"
#include "common.h"
#include "hl7800_client.h"
#include "hl7800_udp.h"
#include "hl7800_http.h"
#include "hl7800_modem.h"
#include "hl7800_serial.h"
#include "hl7800_leaf.h"
#include "app_config.h"

#include <Wire.h>

HL7800Client client_hl78;
HL7800Udp udp_hl78;
HL7800Http http_hl78;
HL7800Modem modem_hl78;
HL7800Leaf leaf_hl78;
extern HL7800Serial serial_hl78;

enum PdpConnectState { 
  RESET,
  INIT,
  CONNECTED,
  FAIL
}; 
PdpConnectState ConnectState = RESET; 

/**
 * LTE 自動接続タスク<br>
 * モデムの初期化からPDP接続までを自動的に行う
 * @param なし
 * @return  true:接続中, false:失敗
 */
bool connect_run(void) {
  if (ConnectState == RESET) {
    if (!modem_hl78.Init()) {
      // モデムからの応答がない場合はエラー終了
      DBGSERIAL.println("@@@@@@ Modem not resoponed!");
      return false;
    }

    // SIM PINチェック
    if (!modem_hl78.chkPin()) {
      // 利用可能なSIMが見つからない場合はエラー終了
      DBGSERIAL.print("PIN locked!");
      ConnectState = RESET;  // 初期化からやり直す
      return false;
    }
    // モデム初期化OK
    ConnectState = INIT;
  }

  // RSSIレベルをチェックし在圏確認
  int rssi = modem_hl78.getRssi();
  if (rssi == 99) {
    // 圏外の場合はエラー終了としし再試行させる
    DBGSERIAL.print("invalid RSSI = ");
    DBGSERIAL.println(rssi);
    return false;
  }
  DBGSERIAL.print("RSSI = ");
  DBGSERIAL.println(rssi);

  // ネットワーク未接続の場合はPDPアクティベートする
  if (!modem_hl78.isActive()) {
    DBGSERIAL.println("PDP connectiong ..");
    if (modem_hl78.activatePdp(PDP_APN, PDP_USER, PDP_PASSWORD, MODEM_CONNECT_TIMEOUT)) {
      DBGSERIAL.println("PDP Connected");
      ConnectState = CONNECTED;
    } else {
      // ネットワーク接続失敗の場合はエラー終了し再試行させる     
      DBGSERIAL.println("PDP Connection Failed!");
      ConnectState = RESET;  // 初期化からやり直す
      return false;
    }
  }

  // 自IPアドレスをチェックしてネットワーク死活チェック
  if (!modem_hl78.chkIpAddress()) {
   // 自IPが不正の場合はPDP切断したうえでエラー終了し、モデムリセットから再試行      
    DBGSERIAL.println("Invalid IP -- PDP disconnect!");
    modem_hl78.deactivate();
    ConnectState = RESET;  // 初期化からやり直す
    return false;
  }

  return true;
}

char HttpBuff[RCVBUFF_SIZE]; // HTTP テストで使う受信メッセージバッファ
/**
 * HTTPでデータを取得(GET)<br>
 * @param なし
 * @return なし
 */
void http_get_test() {
  DBGSERIAL.println("@@@@@ test: HTTP GET @@@@@@@@@@@@@@@@@");
  byte server[] = { TEST_SERVER };
  memset(HttpBuff,0x0,RCVBUFF_SIZE); 
  if (!http_hl78.connect(server,HTTP_PORT, USE_HTTP, NULL, NULL)) {
//  if (!http_hl78.connect(TEST_URL, HTTP_PORT, USE_HTTP, NULL, NULL)) {
    DBGSERIAL.println("@@@@@@ HTTP connection failed!");
  } else {
    DBGSERIAL.println("@@@@@@ HTTP connected");
    int resp = http_hl78.get("/get37.txt", HttpBuff, false);
    DBGSERIAL.print("@@@@@@ HTTP Resp = ");
    DBGSERIAL.println(resp);
    if (resp > 0) {
      DBGSERIAL.println("@@@@@@ HTTP data");
      DBGSERIAL.println("======================================");
      DBGSERIAL.println(HttpBuff);
      DBGSERIAL.println("======================================");
    }
    http_hl78.close();
  }
}

/**
 * HTTPでデータを取得(GET+BASIC認証)<br>
 * @param なし
 * @return なし
 */
#define AUTH_USER "user1"
#define AUTH_PASSWORD "123456"
void http_auth_get_test() {
  DBGSERIAL.println("@@@@@ test: HTTP GET + BASIC-AUTH @@@@");
  byte server[] = { TEST_SERVER };
  memset(HttpBuff,0x0,RCVBUFF_SIZE); 
//  if (!http_hl78.connect(server,HTTP_PORT, USE_HTTP, AUTH_USER, AUTH_PASSWORD )) {
  if (!http_hl78.connect(TEST_URL, HTTP_PORT, USE_HTTP, AUTH_USER, AUTH_PASSWORD)) {
    DBGSERIAL.println("@@@@@@ HTTP connection failed!");
  } else {
    DBGSERIAL.println("@@@@@@ HTTP connected");
    int resp = http_hl78.get("/auth-basic/index.html", HttpBuff, false);
    DBGSERIAL.print("@@@@@@ HTTP Resp = ");
    DBGSERIAL.println(resp);
    if (resp > 0) {
      DBGSERIAL.println("@@@@@@ HTTP data");
      DBGSERIAL.println("======================================");
      DBGSERIAL.println(HttpBuff);
      DBGSERIAL.println("======================================");
    }
    http_hl78.close();
  }
}

/**
 * HTTPでデータを取得(HTTPS GET)<br>
 * @param なし
 * @return なし
 */
void https_get_test() {
  DBGSERIAL.println("@@@@@ test: HTTPS GET");
  byte server[] = { TEST_SERVER };
  memset(HttpBuff,0x0,RCVBUFF_SIZE); 
  if (!http_hl78.connect(server,HTTP_PORT, USE_HTTP, NULL, NULL)) {
//  if (!http_hl78.connect("www.google.co.jp", HTTPS_PORT, USE_HTTPS, NULL, NULL)) {
    DBGSERIAL.println("@@@@@@ HTTP connection failed!");
  } else {
    DBGSERIAL.println("@@@@@@ HTTP connected");
    int resp = http_hl78.get("/get37.txt", HttpBuff,false);
    DBGSERIAL.print("@@@@@@ HTTP Resp = ");
    DBGSERIAL.println(resp);
    if (resp > 0) {
      DBGSERIAL.println("@@@@@@ HTTP data");
      DBGSERIAL.println("======================================");
      DBGSERIAL.println(HttpBuff);
      DBGSERIAL.println("======================================");
    }
    http_hl78.close();
  }
}

/**
 * HTTPでデータを取得(HEAD)<br>
 * @param なし
 * @return なし
 */
// HTTPでデータを取得(HEAD)
void http_head_test() {
  DBGSERIAL.println("@@@@@ test: HTTP HEAD @@@@@@@@@@@@@@@@");
  byte server[] = { TEST_SERVER }; 
  memset(HttpBuff,0x0,RCVBUFF_SIZE); 
  if (!http_hl78.connect(server,HTTP_PORT, USE_HTTP, NULL, NULL)) {
//  if (!http_hl78.connect(TEST_URL, HTTP_PORT, USE_HTTP, NULL, NULL)) {
    DBGSERIAL.println("@@@@@@ HTTP connection failed!");
  } else {
    DBGSERIAL.println("@@@@@@ HTTP connected");
    int resp = http_hl78.head("/get37.txt", HttpBuff);
    DBGSERIAL.print("@@@@@@ HTTP Resp = ");
    DBGSERIAL.println(resp);
    if (resp > 0) {
      DBGSERIAL.println("@@@@@@ HTTP data");
      DBGSERIAL.println("======================================");
      DBGSERIAL.println(HttpBuff);
      DBGSERIAL.println("======================================");
    }
    http_hl78.close();
  }
}

/**
 * HTTPでデータ送信/取得(POST/GET)<br>
 * @param なし
 * @return なし
 */
void http_post_test() {
  DBGSERIAL.println("@@@@@ test: HTTP POST(GET) @@@@@@@@@@@");
  byte server[] = { TEST_SERVER }; 
  memset(HttpBuff,0x0,RCVBUFF_SIZE); 
  if (!http_hl78.connect(server,HTTP_PORT, USE_HTTP, NULL, NULL)) {
//  if (!http_hl78.connect(TEST_URL, HTTP_PORT, USE_HTTP, NULL, NULL)) {
    DBGSERIAL.println("@@@@@@ HTTP connection failed!");
  } else {
    DBGSERIAL.println("@@@@@@ HTTP connected");
    int resp = http_hl78.get("/api.php?num=10", HttpBuff, false);
//    int resp = http_hl78.post("/api.php?num=10", HttpBuff, false);
    DBGSERIAL.print("@@@@@@ HTTP Resp = ");
    DBGSERIAL.println(resp);
    if (resp > 0) {
      DBGSERIAL.println("@@@@@@ HTTP data");
      DBGSERIAL.println("======================================");
      DBGSERIAL.println(HttpBuff);
      DBGSERIAL.println("======================================");
    }
    http_hl78.close();
  }
}

/**
 * Arduinoアプリ初期化<br>
 * @param なし
 * @return なし
 */
void setup() {
  // put your setup code here, to run once:
  ConnectState = RESET; 

  DBGSERIAL.begin(DBGBPS);
#ifdef USE_USBSERIAL
  while (!DBGSERIAL) {
    ; // wait for USB port to connect. Needed for Arduino M0 only
  }
#endif
  delay(1000);
  DBGSERIAL.println("startup...");

  Wire.begin();
  leaf_hl78.setLeafPower(true); // モデムの電源ON
  delay(1000);

  pinMode(LED_BUILTIN, OUTPUT);
}

/**
 * Arduinoアプリタスク<br>
 * @param なし
 * @return なし
 */
void loop() {
  // put your main code here, to run repeatedly:
  DBGSERIAL.println("@@@@ loop() start");
  digitalWrite(LED_BUILTIN, HIGH);

  // LTE 自動接続タスク
  if (!connect_run()) {
    // 失敗時はloop()から抜けて再試行
    digitalWrite(LED_BUILTIN, LOW);
    delay(1000);
    return;
  }

  // 通信処理
  http_get_test();
//  http_auth_get_test();
//  https_get_test();
//  http_head_test();
//  http_post_test();

  // 通信終了後はPDP接続を切断
  ConnectState = INIT;
  modem_hl78.deactivate();
 
  digitalWrite(LED_BUILTIN, LOW);
  DBGSERIAL.println("@@@@ loop()end");
  delay(30000); // データを流しすぎないよう待機

}
