/**
 * @file IFTTT_Demo1.ino
 * @brief IFTTT Webhocksに向けてバッテリ電圧値を送信 <br>
 * 本スケッチを利用するにあたりAVRマイコンリーフでは"recive data is too big!"というログが出る場合があります<BR>
 * その場合は受信バッファを増量してください<BR>
 * ライブラリ内のcommon.h にある #define RCVBUFF_SIZE 100 を 400に変更
 * 
 * @date  2019/10/25 Preview-Beta
 * @author kt-nakamura@kddi-tech.com
 */
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
#include "Arduino.h"
#include "common.h"
#include "hl7800_client.h"
#include "hl7800_udp.h"
#include "hl7800_modem.h"
#include "hl7800_serial.h"
#include "hl7800_leaf.h"
#include "app_config.h"

#include <Wire.h>

PROGMEM const char IFTTT_KEY_STR[]=IFTTT_KEY;
PROGMEM const char IFTTT_TRG_STR[]=IFTTT_TRIGGER;

#if defined(__AVR__) // AVR leaf
 #include <avr/wdt.h>
 #include <avr/sleep.h>
#else
 #include <Adafruit_SleepyDog.h> // DeepSleep https://github.com/adafruit/Adafruit_SleepyDog
#endif

HL7800Client client_hl78;
HL7800Udp udp_hl78;
HL7800Modem modem_hl78;
HL7800Leaf leaf_hl78;
extern HL7800Serial serial_hl78;

enum PdpConnectState { 
  RESET,
  INIT,
  CONNECTED,
  FAIL
}; 
PdpConnectState ConnectState = RESET; 

#if defined(__AVR__) // AVR leaf 専用コード
/**********************************************
* DeepSleep(AVR)
**********************************************/
void avr_sleep(){ 
  ADCSRA &= ~(1 << ADEN);                 //ADC停止
  PRR = 0xE4;
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);    //SET SLEEP MODE
  sleep_enable();                         // SLEEP ENABLE

  // BOD停止
  MCUCR |= (1 << BODSE) | (1 << BODS);             // MCUCRのBODSとBODSEに1をセット
  MCUCR = (MCUCR & ~(1 << BODSE)) | (1 << BODS);  // すぐに（4クロック以内）BODSSEを0, BODSを1に設定
  asm("sleep");                                   // 3クロック以内にスリープ 
  sleep_disable();                         // SLEEP DISABLE
  PRR = 0x05;
}
//-----------------------------------------
// WDT
//-----------------------------------------
/**********************************************
* WDT割り込み
**********************************************/
ISR(WDT_vect){
  wdt_disable();
}
/**********************************************
* WDTスタート
**********************************************/
void avr_wdt_start(){
  // watchdog timer reset
  wdt_reset();  
  //disable interruput
  cli();
  //clear WatchDog system Reset Flag(WDRF)
  MCUSR &= ~(1 << WDRF);
  // WDT変更許可
  // WDCEとWDE同時セットで変更許可
  WDTCSR |= 1 << WDCE | 1 << WDE;
  //WDT設定
  // WDE=0,WDIE=1 :WDT overflowで割り込み
  // WDP3=1,WDP2=0,WDP1=0,WDP0=1: 8s
  WDTCSR = 1 << WDIE | 0 << WDE | 1 << WDP3 | 0 << WDP2 | 0 << WDP1 | 1 << WDP0;
  //enable interruput
  sei();
}
#endif

/**
 * LTE 自動接続タスク<br>
 * モデムの初期化からPDP接続までを自動的に行う
 * @param なし
 * @return  true:接続中, false:失敗
 */
bool connect_run(void) {
  if (ConnectState == RESET) {
    if (!modem_hl78.Init()) {
      // モデムからの応答がない場合はエラー終了
      DBGSERIAL.println("@@@@@@ Modem not resoponed!");
      return false;
    }

    // SIM PINチェック
    if (!modem_hl78.chkPin()) {
      // 利用可能なSIMが見つからない場合はエラー終了
      DBGSERIAL.print("PIN locked!");
      ConnectState = RESET;  // 初期化からやり直す
      return false;
    }
    // モデム初期化OK
    ConnectState = INIT;
  }

  // モデムの応答チェック
  if (!modem_hl78.waitModem()){
    // モデムからの応答がない場合はエラー終了
    ConnectState = RESET;  // 初期化からやり直す
    return false;
  }

  // RSSIレベルをチェックし在圏確認
  for (int i=0;i<10;i++) {
    int rssi = modem_hl78.getRssi();
    DBGSERIAL.print("RSSI = ");
    DBGSERIAL.println(rssi);
    if (rssi != 99) {
      i=11;
    }
    delay(1000);
  }

  // ネットワーク未接続の場合はPDPアクティベートする
  if (!modem_hl78.isActive()) {
    DBGSERIAL.println("PDP connectiong ..");
    if (modem_hl78.activatePdp(PDP_APN, PDP_USER, PDP_PASSWORD, MODEM_CONNECT_TIMEOUT)) {
      DBGSERIAL.println("PDP Connected");
      ConnectState = CONNECTED;
    } else {
      // ネットワーク接続失敗の場合はエラー終了し再試行させる     
      DBGSERIAL.println("PDP Connection Failed!");
      ConnectState = RESET;  // 初期化からやり直す
      return false;
    }
  }

  // 自IPアドレスをチェックしてネットワーク死活チェック
  if (!modem_hl78.chkIpAddress()) {
   // 自IPが不正の場合はPDP切断したうえでエラー終了し、モデムリセットから再試行      
    DBGSERIAL.println("Invalid IP -- PDP disconnect!");
    modem_hl78.deactivate();
    ConnectState = RESET;  // 初期化からやり直す
    return false;
  }

  return true;
}

/**
 * IFTTT送信（HTTP)<br>
 * @param 送信データ
 * @return なし
 */
bool ifttt_send_http(int mvolt) {
  DBGSERIAL.println("@@@@@ test: TCP HTTP-GET @@@@@@@@@@@@@");
  char uri[200];

  // TCPソケットをオープン
  if (!client_hl78.connect("maker.ifttt.com",80)) {
    DBGSERIAL.println("TCP connection failed!");
    return false;
  } else {
    DBGSERIAL.println("TCP connected");

//    sprintf_P(uri,PSTR("GET /trigger/%s/with/key/%s?value1=%d HTTP/1.1\r\nHost: maker.ifttt.com\r\nConnection: Keep-Alive\r\n\r\n"), IFTTT_TRG_STR, IFTTT_KEY_STR, mvolt); // Keep-Aliveをしてしないと最終パケットでTCP切断されデータが受け取れない
    sprintf_P(uri,PSTR("GET /trigger/%s/with/key/%s?value1=%d HTTP/1.1\r\nHost: maker.ifttt.com\r\nConnection: close\r\n\r\n"), IFTTT_TRIGGER, IFTTT_KEY, mvolt); // Keep-Aliveをしてしないと最終パケットでTCP切断されデータが受け取れない
//    sprintf_P(uri,PSTR("GET /trigger/%s/with/key/%s?value1=%d HTTP/1.1\r\nHost: maker.ifttt.com\r\nConnection: close\r\n\r\n"), IFTTT_TRG_STR, IFTTT_KEY_STR, mvolt);

//    DBGSERIAL.println(uri);

    // TCP 送信
    if (!client_hl78.print(uri)) {
      DBGSERIAL.println("TCP write failed");
      return false;
    } else {
      // TCP 受信
      int resp_byte = client_hl78.available();
      for (int i=0;i<resp_byte;i++) {
        char c = client_hl78.read();
#if 0
        DBGSERIAL.write(c);
#endif
      }
    }

    // TCPソケットをクローズ
    client_hl78.stop();
  }

  return true;
}


/**
 * Arduinoアプリ初期化<br>
 * @param なし
 * @return なし
 */
void setup() {
  // put your setup code here, to run once:
  ConnectState = RESET; 

  DBGSERIAL.begin(DBGBPS);
#ifdef USE_USBSERIAL
//  while (!DBGSERIAL) {
//    ; // wait for USB port to connect. Needed for Arduino M0 only
//  }
#endif
  delay(1000);
  DBGSERIAL.println("startup...");

  Wire.begin();
  leaf_hl78.setLeafPower(true); // モデムの電源ON
  delay(1000);

  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, HIGH);

#if !defined(__AVR__) //M0 leaf
  // 未使用端子を固定する
  pinMode(0, OUTPUT);
  digitalWrite(0, LOW);
  pinMode(1, OUTPUT);
  digitalWrite(1, LOW);
  pinMode(2, OUTPUT);
  digitalWrite(2, LOW);
  pinMode(6, OUTPUT);
  digitalWrite(6, LOW);
  pinMode(7, OUTPUT);
  digitalWrite(7, LOW);
#endif
}

/**
 * Arduinoアプリタスク<br>
 * @param なし
 * @return なし
 */
void loop() {
  // put your main code here, to run repeatedly:
  DBGSERIAL.println("@@@@ loop() start");
  digitalWrite(LED_BUILTIN, HIGH);
  delay(5000);
#if 1
  // バッテリ電圧取得
  int mvolt =  leaf_hl78.getLeafADC();
  DBGSERIAL.print("batt=");
  DBGSERIAL.print(mvolt);
  DBGSERIAL.println("mV");

  // LTE 自動接続タスク
  if (connect_run()) {
    // SORACOM　binary parserへデータ送信
    if (ifttt_send_http(mvolt)) {
      ConnectState = INIT;
    } else {
      ConnectState = RESET; // 通信失敗時は次回初期化から開始させる
    }

    // テスト成功後はPDP接続を切断
    modem_hl78.deactivate();
  }

#endif

  // 省電力状態への移行
  DBGSERIAL.println("Enter Sleep");
  digitalWrite(LED_BUILTIN, LOW);

#ifdef USE_SLEEP
  // モデムを省電力状態に移行
modem_hl78.enterSleep(HIBERNATE);
//modem_hl78.enterSleep(LIGHT_HIBERNATE);
//modem_hl78.enterSleep(SLEEP_MODE);
#endif

#ifdef USE_POWERDOWN
  // モデムパワーダウン
  modem_hl78.powerDown();
  ConnectState = RESET;
#endif

#ifdef USE_POWEROFF
  // モデム電源断
  leaf_hl78.setLeafPower(false);
  ConnectState = RESET;
#endif

#if defined(__AVR__) // AVR leaf
  DBGSERIAL.println("AVR Sleep");
  delay(1000);
  for (int i=0; i < 75 ; i++) { // 75*8000ms = 10min
    avr_wdt_start(); // 8sec
    avr_sleep();          
  }
#else
  DBGSERIAL.println("M0 Sleep");
  delay(1000);
  pinMode(0 , OUTPUT);  // Serial-RXピンから電流漏れがあるため固定する
  digitalWrite(0,LOW); 
//  for (int i=0; i < 8 ; i++) { // 8*8000ms = 1.067min
  for (int i=0; i < 75 ; i++) { // 75*8000ms = 10min
    int sleepMS = Watchdog.sleep(8000); // valid param 1000ms,2000ms,4000ms,8000ms,16000ms
  }
  USBDevice.attach();
#endif

  // 省電力状態からの復帰
  DBGSERIAL.println("Resume Sleep");

#ifdef USE_SLEEP
  // モデムの省電力状態を解除
  modem_hl78.leaveSleep();
#endif

#ifdef USE_POWEROFF
  // モデム電源オン
  leaf_hl78.setLeafPower(true);
#endif

#ifdef USE_POWERDOWN
  // モデム電源復帰
  modem_hl78.powerUp();
#endif

//  delay(30000); // データを流しすぎないよう待機
  DBGSERIAL.println("@@@@ loop() end");
}
