/**
 * @file ModemApiSample.ino
 * @brief HL7800 Library test-app <br>
 * 
 * @date  2019/9/17 Preview-Bata
 * @author kt-nakamura@kddi-tech.com
 */
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
#include "Arduino.h"
#include "common.h"
#include "hl7800_client.h"
#include "hl7800_udp.h"
#include "hl7800_http.h"
#include "hl7800_modem.h"
#include "hl7800_serial.h"
#include "hl7800_leaf.h"
#include "app_config.h"

#include <Wire.h>

HL7800Client client_hl78;
HL7800Udp udp_hl78;
HL7800Http http_hl78;
HL7800Modem modem_hl78;
HL7800Leaf leaf_hl78;
extern HL7800Serial serial_hl78;

enum PdpConnectState { 
  RESET,
  INIT,
  CONNECTED,
  FAIL
}; 
PdpConnectState ConnectState = RESET; 

/**
 * Arduinoアプリ初期化<br>
 * @param なし
 * @return なし
 */
void setup() {
  // put your setup code here, to run once:
  ConnectState = RESET; 

  DBGSERIAL.begin(DBGBPS);
#ifdef USE_USBSERIAL
  while (!DBGSERIAL) {
    ; // wait for USB port to connect. Needed for Arduino M0 only
  }
#endif
  delay(1000);
  DBGSERIAL.println("startup...");

  Wire.begin();
  leaf_hl78.setLeafPower(true); // モデムの電源ON
  delay(1000);

  pinMode(LED_BUILTIN, OUTPUT);
}

/**
 * Arduinoアプリタスク<br>
 * @param なし
 * @return なし
 */
void loop() {
  // put your main code here, to run repeatedly:
  DBGSERIAL.println("@@@@ loop() start");

  if (ConnectState == RESET) {
    if (!modem_hl78.Init()) {
      // モデムからの応答がない場合はエラー終了
      DBGSERIAL.println("@@@@@@ Modem not resoponed!");
      delay(1000);
      return ;
    }
    ConnectState = INIT;
  }

  // バッテリ電圧取得
  int mvolt =  leaf_hl78.getLeafADC();
  DBGSERIAL.print("batt=");
  DBGSERIAL.print(mvolt);
  DBGSERIAL.println("mV");
  delay(1000);

  // IMEI取得チェック
  char imei[20];
  if (modem_hl78.getImei(imei) > 0) {
    DBGSERIAL.print("IMEI = ");
    DBGSERIAL.println(imei);
  }
  // CCID取得
  char ccid[20];
  if (modem_hl78.getCcid(ccid) > 0) {
    DBGSERIAL.print("CCID = ");
    DBGSERIAL.println(ccid);
  }

  // SIM PINチェック
  if (!modem_hl78.chkPin()) {
    DBGSERIAL.print("PIN locked!");
  }

  // RSSIレベルをチェックし在圏確認
  int rssi = modem_hl78.getRssi();
  DBGSERIAL.print("RSSI = ");
  DBGSERIAL.println(rssi);

  char time[30];
  if (modem_hl78.getRTC(time) > 0) {
    DBGSERIAL.print("RTC TIME = ");
    DBGSERIAL.println(time);
  }

  DBGSERIAL.println("@@@@ loop() end");
  delay(3000);
 
}
