/**
 * @file HL7800_TcpSample.ino
 * @brief HL7800 Library test-app <br>
 * 
 * @date  2019/9/17 Preview-Bata
 * @author kt-nakamura@kddi-tech.com
 */
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
#include "Arduino.h"
#include "common.h"
#include "hl7800_client.h"
#include "hl7800_udp.h"
#include "hl7800_http.h"
#include "hl7800_modem.h"
#include "hl7800_serial.h"
#include "hl7800_leaf.h"
#include "app_config.h"

#include <Wire.h>

HL7800Client client_hl78;
HL7800Udp udp_hl78;
HL7800Http http_hl78;
HL7800Modem modem_hl78;
HL7800Leaf leaf_hl78;
extern HL7800Serial serial_hl78;

enum PdpConnectState { 
  RESET,
  INIT,
  CONNECTED,
  FAIL
}; 
PdpConnectState ConnectState = RESET; 

/**
 * LTE 自動接続タスク<br>
 * モデムの初期化からPDP接続までを自動的に行う
 * @param なし
 * @return  true:接続中, false:失敗
 */
bool connect_run(void) {
  if (ConnectState == RESET) {
    if (!modem_hl78.Init()) {
      // モデムからの応答がない場合はエラー終了
      DBGSERIAL.println("@@@@@@ Modem not resoponed!");
      return false;
    }

    // SIM PINチェック
    if (!modem_hl78.chkPin()) {
      // 利用可能なSIMが見つからない場合はエラー終了
      DBGSERIAL.print("PIN locked!");
      ConnectState = RESET;  // 初期化からやり直す
      return false;
    }
    // モデム初期化OK
    ConnectState = INIT;
  }

  // RSSIレベルをチェックし在圏確認
  int rssi = modem_hl78.getRssi();
  if (rssi == 99) {
    // 圏外の場合はエラー終了としし再試行させる
    DBGSERIAL.print("invalid RSSI = ");
    DBGSERIAL.println(rssi);
    return false;
  }
  DBGSERIAL.print("RSSI = ");
  DBGSERIAL.println(rssi);

  // ネットワーク未接続の場合はPDPアクティベートする
  if (!modem_hl78.isActive()) {
    DBGSERIAL.println("PDP connectiong ..");
    if (modem_hl78.activatePdp(PDP_APN, PDP_USER, PDP_PASSWORD, MODEM_CONNECT_TIMEOUT)) {
      DBGSERIAL.println("PDP Connected");
      ConnectState = CONNECTED;
    } else {
      // ネットワーク接続失敗の場合はエラー終了し再試行させる     
      DBGSERIAL.println("PDP Connection Failed!");
      ConnectState = RESET;  // 初期化からやり直す
      return false;
    }
  }

  // 自IPアドレスをチェックしてネットワーク死活チェック
  if (!modem_hl78.chkIpAddress()) {
   // 自IPが不正の場合はPDP切断したうえでエラー終了し、モデムリセットから再試行      
    DBGSERIAL.println("Invalid IP -- PDP disconnect!");
    modem_hl78.deactivate();
    ConnectState = RESET;  // 初期化からやり直す
    return false;
  }

  return true;
}

/**
 * TCPsocketテスト(HTTP GET)<br>
 * @param なし
 * @return なし
 */
void socket_http_get() {
  DBGSERIAL.println("@@@@@ test: TCP HTTP-GET @@@@@@@@@@@@@");
  // TCPソケットをオープン
  byte server[] = { TEST_SERVER}; 
  if (!client_hl78.connect(server,80)) {
//  if (!client_hl78.connect(TEST_URL,80)) {
    DBGSERIAL.println("TCP connection failed!");
  } else {
    DBGSERIAL.println("TCP connected");

    // TCP 送信
    if (!client_hl78.print("GET /get37.txt HTTP/1.0\r\nConnection: Keep-Alive\r\n\r\n")) { // Keep-Aliveをしてしないと最終パケットで切断されデータが受け取れない
      DBGSERIAL.println("TCP write failed");
    } else {
      // TCP 受信
      DBGSERIAL.println("======================================");
      while (client_hl78.available() > 0) { // HTTPレスポンスヘッダを意識していないためモデムのTCP切断で終了
        char c = client_hl78.read();
        DBGSERIAL.write(c);
      }
      DBGSERIAL.println("======================================");
    }
    // TCPソケットをクローズ
    client_hl78.stop();
  }
}

/**
 * TCPsocketテスト(HTTP GET+データ送信のみ)<br>
 * @param なし
 * @return なし
 */
void socket_http_send() {
  DBGSERIAL.println("@@@@@ test: TCP HTTP-GET/SEND @@@@@@@@");
  // TCPソケットをオープン
  byte server[] = { TEST_SERVER}; 
  if (!client_hl78.connect(server,80)) {
//  if (!client_hl78.connect(TEST_URL,80)) {
    DBGSERIAL.println("TCP connection failed!");
  } else {
    DBGSERIAL.println("TCP connected");

    // TCP 送信
    if (!client_hl78.print("GET /api.php?num=10 HTTP/1.0\r\n\r\n")) {
      DBGSERIAL.println("TCP write failed");
    } else {
      // TCP 受信(HTTPレスポンスヘッダ+ボディ)
      DBGSERIAL.println("======================================");
      while (client_hl78.available() > 0) { // HTTPレスポンスヘッダを意識していないためモデムのTCP切断で終了
        char c = client_hl78.read();
        DBGSERIAL.write(c);
      }
      DBGSERIAL.println("======================================");
    }
    client_hl78.stop();
  }
}

/**
 * Arduinoアプリ初期化<br>
 * @param なし
 * @return なし
 */
void setup() {
  // put your setup code here, to run once:
  ConnectState = RESET; 

  DBGSERIAL.begin(DBGBPS);
#ifdef USE_USBSERIAL
  while (!DBGSERIAL) {
    ; // wait for USB port to connect. Needed for Arduino M0 only
  }
#endif
  delay(1000);
  DBGSERIAL.println("startup...");

  Wire.begin();
  leaf_hl78.setLeafPower(true); // モデムの電源ON
  delay(1000);

  pinMode(LED_BUILTIN, OUTPUT);
}

/**
 * Arduinoアプリタスク<br>
 * @param なし
 * @return なし
 */
void loop() {
  // put your main code here, to run repeatedly:
  DBGSERIAL.println("@@@@ loop() start");
  digitalWrite(LED_BUILTIN, HIGH);

  // LTE 自動接続タスク
  if (!connect_run()) {
    // 失敗時はloop()から抜けて再試行
    digitalWrite(LED_BUILTIN, LOW);
    delay(1000);
    return;
  }

  // TCP送受信
  socket_http_get();
//  socket_http_send();

  // テスト成功後はPDP接続を切断
  ConnectState = INIT;
  modem_hl78.deactivate();
 
  digitalWrite(LED_BUILTIN, LOW);
  DBGSERIAL.println("@@@@ loop() end");
  delay(30000); // データを流しすぎないよう待機

}
