/**
 * @file hl7800_leaf.cpp
 * @brief HL7800 LTE-M Leaf  API <br>
 * 
 * @date  2019/9/11 Preview-Bata
 * @author kt-nakamura@kddi-tech.com
 */

#include "hl7800_leaf.h"
#include "common.h"
#include <Wire.h>

/**
 * バッテリ電圧読み出し<br>
 * Leafに接続されたバッテリの電圧をmV単位で読み出す<br>
 * 
 * @param なし
 * @return バッテリ電圧(mV)
 */
 int HL7800Leaf::getLeafADC(void) {
  Wire.requestFrom(LEAF_ADC_I2C, 2);
  byte val[2];
  int i = 0;
  while (Wire.available()) {
    val[i] = Wire.read();
    i++;
  }
  int adc = (val[0] << 4) + (val[1]>>4);
//  int volt = (adc * 3300 * 2) >> 8 ;
  int volt = (adc * 26) ; // 3300*2/256 = 25.78125
  return volt;  
}

/**
 * モデム電源制御<br>
 * モデムのPBATT入力に対して電源ON/OFFを制御する<br>
 * 
 * @param enable false:電源OFF,true:電源ON(default)
 * @return なし
 */
void HL7800Leaf::setLeafPower(bool enable) {
  Wire.beginTransmission(LEAF_GPIO_I2C);
  Wire.write(1);  // Output Port reg
  if (enable) {
    DBGSERIAL.println(F("HL7800 PoweON"));  
    Wire.write(0);  // port2 -- HL7800_OFF1
  } else {
    DBGSERIAL.println(F("HL7800 PoweOFF"));  
    Wire.write(2);  // port2 -- HL7800_OFF1
  }
  Wire.endTransmission();

  Wire.beginTransmission(LEAF_GPIO_I2C);
  Wire.write(3);  // Configuration reg
  Wire.write(0xfd);  // port2 -- HL7800_OFF1
  Wire.endTransmission();
}
