/**
 * @file hl7800_udp.h
 * @brief  HL7800 modem UDP client API <br>
 * 
 * @date  2019/9/5 Preview-Alpha
 * @author kt-nakamura@kddi-tech.com
 */

#ifndef HL7800_UDP_H
#define HL7800_UDP_H

#include "Arduino.h"
#include "common.h"
#include <Udp.h>
#include "IPAddress.h"

#define MAX_HOSTNAME_SIZE 20

class HL7800Udp : public UDP {
public:
//  HL7800Udp();  // Constructor
  virtual uint8_t begin(uint16_t);
  virtual void stop();  // Finish with the UDP socket
  virtual int beginPacket(IPAddress ip, uint16_t port);
  virtual int beginPacket(const char *host, uint16_t port);
  virtual int endPacket();
  virtual size_t write(uint8_t);
  virtual size_t write(const uint8_t *buffer, size_t size);
  virtual int parsePacket();
  virtual int available();
  virtual int read();
  virtual int read(unsigned char* buffer, size_t len);
  virtual int read(char* buffer, size_t len) { return read((unsigned char*)buffer, len); };
  virtual int peek();
  virtual void flush(); // Finish reading the current packet
  virtual IPAddress remoteIP() { return _remoteIP; };
  virtual uint16_t remotePort() { return _remotePort; };

private:
  char cmd[20];
//  uint8_t _sock;  // socket ID for Wiz5100
  uint16_t _port; // local port to listen on
  IPAddress _remoteIP; // remote IP address for the incoming packet whilst it's being processed
  uint16_t _remotePort; // remote port for the incoming packet whilst it's being processed
  uint16_t _offset; // offset into the packet being sent
  uint16_t _remaining; // remaining bytes of incoming packet yet to be processed

  char _targetHost[MAX_HOSTNAME_SIZE];
  IPAddress _targetIP;
  uint16_t _targetPort;
  bool _validHost;
  bool _validIP;

};

#endif // HL7800_UDP_H
